-----------------------------------------------------------------------------------------------
-- TikiToy by TippingPoint of
-- Internecion - Dominion@Bloodsworn
-- 
--
-- Based on the original Walatiki Timers by Shikaga
-- with updates to improve visibility, loadtimes, 
-- formatting and structure.
--
-- Usage: /itt (Will display window.)
-----------------------------------------------------------------------------------------------
 
require "Window"
 
-----------------------------------------------------------------------------------------------
-- TikiToy Timer Module Definition
-----------------------------------------------------------------------------------------------
local TikiToy= {}
 
-----------------------------------------------------------------------------------------------
-- Instantiation
-----------------------------------------------------------------------------------------------
function TikiToy:new(o)
    o = o or {}
    setmetatable(o, self)
    self.__index = self 
    return o
end

-----------------------------------------------------------------------------------------------
-- Initialization 
-----------------------------------------------------------------------------------------------
function TikiToy:Init()
	local bHasConfigureFunction = false
	local strConfigureButtonText = "TikiToy"
	local tDependencies = {} 
    Apollo.RegisterAddon(self, bHasConfigureFunction, strConfigureButtonText, tDependencies)
end


-----------------------------------------------------------------------------------------------
-- Handling for slash commands. Currently is /itt
-----------------------------------------------------------------------------------------------
function TikiToy:OnTTShow()
	if self.wndMain:IsShown(false) then
---		Apollo.AddAddonErrorText(self, "Found window, closing!")
		self.wndMain:Show(false)
	else
---		Apollo.AddAddonErrorText(self, "Didn't find window, opening")
		self.wndMain:Show(true)
	end
end

-----------------------------------------------------------------------------------------------
-- OnLoad event handling for chatmessages and timers, Doc load
-----------------------------------------------------------------------------------------------
function TikiToy:OnLoad()
    Apollo.RegisterEventHandler("ChatMessage","OnChatMessage", self)
	self.xmlDoc = XmlDoc.CreateFromFile("TikiToy.xml")
	self.xmlDoc:RegisterCallback("OnDocLoaded", self)
end

function TikiToy:OnDocLoaded()
	if self.xmlDoc ~= nil and self.xmlDoc:IsLoaded() then
		self.wndMain = Apollo.LoadForm(self.xmlDoc, "ITTMainwnd", nil, self)
		if self.wndMain == nil then
			Apollo.AddAddonErrorText(self, "Could not load the main window for some dumbass reason.")
			return
		end
	end
	
	self.mmtimers = {}
    self.startTime = nil


	Apollo.RegisterTimerHandler("OneSecTimer", "OnTimer", self)
	Apollo.RegisterSlashCommand("itt", "OnTTShow", self) 	
	self.MMunearthedTimeDropped = nil
end

-----------------------------------------------------------------------------------------------
-- Function to easily perform round since Lua is deficient...
-----------------------------------------------------------------------------------------------
function round(num, idp)
  local mult = 10^(idp or 0)
  return math.floor(num * mult + 0.5) / mult
end

-----------------------------------------------------------------------------------------------
-- Function to invoke actual main window class based on the "ITTMainwnd" 
-- Creates the framework for the window load
-----------------------------------------------------------------------------------------------
function TikiToy:Window( ... )
	if Apollo.FindWindowByName("ITTMainwnd") == nil then
		self.wndMain = Apollo.LoadForm(self.xmlDoc, "ITTMainwnd", nil, self)
		self.wndMain:Invoke()
	end
end


-----------------------------------------------------------------------------------------------
-- Instantiates window "TikiToy" and displays children from timer events
-----------------------------------------------------------------------------------------------
function TikiToy:OnTimer()
	self:CheckUnearthedDespawns()
	if self.wndMain then
		self.wndMain:DestroyChildren()
		local timerInstance = Apollo.LoadForm(self.xmlDoc, "ITTtimer", self.wndMain, self)
		timerInstance:FindChild("Timer"):SetText("TikiTimers")
		for _,time in pairs(self.mmtimers) do
			local dTime = round(60 - (GameLib.GetGameTime() - time))
			if dTime > 0 then
				local timerInstance = Apollo.LoadForm(self.xmlDoc, "ITTtimer", self.wndMain, self)
				timerInstance:FindChild("Timer"):SetText(dTime)
			else
			end
		end
		self.wndMain:ArrangeChildrenVert(0)
	else
	end
end


-----------------------------------------------------------------------------------------------
-- Creates a timer based on the drop time of a moodie mask.
-----------------------------------------------------------------------------------------------
function TikiToy:CheckUnearthedDespawns()
	if self.unearthedDropppedTime then
		if (GameLib.GetGameTime() - self.unearthedDropppedTime) > 30 then
			table.insert(self.mmtimers, GameLib.GetGameTime())
			self.MMunearthedTimeDropped = nil
		end
	end
end

function TikiToy:UnearthedDropped()
	self.MMunearthedTimeDropped = GameLib.GetGameTime()
end

function TikiToy:UnearthedPickedUp()
	self.MMunearthedTimeDropped = nil
end
 
function TikiToy:OnChatMessage(channelCurrent, tMessage)
	if channelCurrent:GetName() == "Datachron" then
		local message = tMessage.arMessageSegments[1].strText
		if message == "The Dominion captured a Moodie Mask!" or message == "The Exiles captured a Moodie Mask!" then
			table.insert(self.mmtimers, GameLib.GetGameTime())
		elseif message == "A Moodie Mask has been unearthed!"  then
			self:Window()
		elseif message == "The Dominion gathered a Moodie Mask!" or message == "The Exiles gathered a Moodie Mask!" then
			self:UnearthedPickedUp()
		elseif message == "The Dominion dropped a Moodie Mask!" or message == "The Exiles dropped a Moodie Mask!" then
			self:UnearthedDropped()
		else
			Print(message)
		end
	end
end

local TikiToyInst = TikiToy:new()
TikiToyInst:Init()
